@@ -308,20 +308,20 @@ class ModelSQL(ModelStorage):
 
         if cls._history:
             cls._update_history_table()
-            history_table = cls.__table_history__()
+            h_table = cls.__table_history__()
             cursor.execute(*sql_table.select(sql_table.id, limit=1))
             if cursor.fetchone():
                 cursor.execute(
-                    *history_table.select(history_table.id, limit=1))
+                    *h_table.select(h_table.id, limit=1))
                 if not cursor.fetchone():
                     columns = [n for n, f in cls._fields.items()
                         if f.sql_type()]
-                    cursor.execute(*history_table.insert(
-                            [Column(history_table, c) for c in columns],
+                    cursor.execute(*h_table.insert(
+                            [Column(h_table, c) for c in columns],
                             sql_table.select(*(Column(sql_table, c)
                                     for c in columns))))
-                    cursor.execute(*history_table.update(
-                            [history_table.write_date], [None]))
+                    cursor.execute(*h_table.update(
+                            [h_table.write_date], [None]))
 
     @classmethod
     def _update_history_table(cls):
@@ -1129,11 +1129,14 @@ class ModelSQL(ModelStorage):
                     Column(foreign_table, field_name), sub_ids)
                 cursor.execute(*foreign_table.select(foreign_table.id,
                         where=foreign_red_sql))
-                records = Model.browse([x[0] for x in cursor])
+                related_records = Model.browse([x[0] for x in cursor])
             else:
                 with transaction.set_context(active_test=False):
-                    records = Model.search([(field_name, 'in', sub_ids)])
-            return records
+                    related_records = Model.search(
+                        [(field_name, 'in', sub_ids)])
+            if Model == cls:
+                related_records = list(set(related_records) - set(records))
+            return related_records
 
         for sub_ids, sub_records in zip(
                 grouped_slice(ids), grouped_slice(records)):
@@ -1147,9 +1150,10 @@ class ModelSQL(ModelStorage):
                 if (not hasattr(Model, 'search')
                         or not hasattr(Model, 'write')):
                     continue
-                records = get_related_records(Model, field_name, sub_ids)
-                if records:
-                    Model.write(records, {
+                related_records = get_related_records(
+                    Model, field_name, sub_ids)
+                if related_records:
+                    Model.write(related_records, {
                             field_name: None,
                             })
 
@@ -1157,9 +1161,10 @@ class ModelSQL(ModelStorage):
                 if (not hasattr(Model, 'search')
                         or not hasattr(Model, 'delete')):
                     continue
-                records = get_related_records(Model, field_name, sub_ids)
-                if records:
-                    Model.delete(records)
+                related_records = get_related_records(
+                    Model, field_name, sub_ids)
+                if related_records:
+                    Model.delete(related_records)
 
             for Model, field_name in foreign_keys_tocheck:
                 with Transaction().set_context(
