@@ -17,7 +17,6 @@ import trytond.tools as tools
 from trytond.config import config
 from trytond.exceptions import MissingDependenciesException
 from trytond.transaction import Transaction
-from trytond import backend
 import trytond.convert as convert
 
 logger = logging.getLogger(__name__)
@@ -80,7 +79,6 @@ def import_module(name, fullname=None):
     return module
 
 
-
 def get_module_info(name):
     "Return the content of the tryton.cfg"
     module_config = configparser.ConfigParser()
@@ -166,7 +164,29 @@ def is_module_to_install(module, update):
     return False
 
 
+def load_translations(pool, node, languages):
+    module = node.name
+    localedir = '%s/%s' % (node.info['directory'], 'locale')
+    lang2filenames = defaultdict(list)
+    for filename in itertools.chain(
+            iglob('%s/*.po' % localedir),
+            iglob('%s/override/*.po' % localedir)):
+        filename = filename.replace('/', os.sep)
+        lang = os.path.splitext(os.path.basename(filename))[0]
+        if lang not in languages:
+            continue
+        lang2filenames[lang].append(filename)
+    base_path_position = len(node.info['directory']) + 1
+    for language, files in lang2filenames.items():
+        filenames = [f[base_path_position:] for f in files]
+        logger.info('%s:loading %s', module, ','.join(filenames))
+        Translation = pool.get('ir.translation')
+        Translation.translation_import(language, module, files)
+
+
 def load_module_graph(graph, pool, update=None, lang=None):
+    # Prevent to import backend when importing module
+    from trytond.cache import Cache
     from trytond.ir.lang import get_parent_language
 
     if lang is None:
@@ -186,9 +206,11 @@ def load_module_graph(graph, pool, update=None, lang=None):
     transaction = Transaction()
     with transaction.connection.cursor() as cursor:
         modules = [x.name for x in graph]
-        cursor.execute(*ir_module.select(ir_module.name, ir_module.state,
-                where=ir_module.name.in_(modules)))
-        module2state = dict(cursor.fetchall())
+        module2state = dict()
+        for sub_modules in tools.grouped_slice(modules):
+            cursor.execute(*ir_module.select(ir_module.name, ir_module.state,
+                    where=ir_module.name.in_(list(sub_modules))))
+            module2state.update(cursor)
         modules = set(modules)
 
         for node in graph:
@@ -220,7 +242,7 @@ def load_module_graph(graph, pool, update=None, lang=None):
 
                 # Instanciate a new parser for the module
                 tryton_parser = convert.TrytondXmlHandler(
-                    pool, module, package_state, modules)
+                    pool, module, package_state, modules, lang)
 
                 for filename in node.info.get('xml', []):
                     filename = filename.replace('/', os.sep)
@@ -231,22 +253,7 @@ def load_module_graph(graph, pool, update=None, lang=None):
 
                 modules_todo.append((module, list(tryton_parser.to_delete)))
 
-                localedir = '%s/%s' % (node.info['directory'], 'locale')
-                lang2filenames = defaultdict(list)
-                for filename in itertools.chain(
-                        iglob('%s/*.po' % localedir),
-                        iglob('%s/override/*.po' % localedir)):
-                    filename = filename.replace('/', os.sep)
-                    lang2 = os.path.splitext(os.path.basename(filename))[0]
-                    if lang2 not in lang:
-                        continue
-                    lang2filenames[lang2].append(filename)
-                base_path_position = len(node.info['directory']) + 1
-                for language, files in lang2filenames.items():
-                    filenames = [f[base_path_position:] for f in files]
-                    logger.info('%s:loading %s', module, ','.join(filenames))
-                    Translation = pool.get('ir.translation')
-                    Translation.translation_import(language, module, files)
+                load_translations(pool, node, lang)
 
                 if package_state == 'to remove':
                     continue
@@ -264,7 +271,13 @@ def load_module_graph(graph, pool, update=None, lang=None):
                                 ]))
                 module2state[module] = 'activated'
 
+            # Clear cache from old data cached before transaction started
+            Cache.clear_all()
+            # Avoid clearing cache to prevent dead lock on ir.cache table
+            Cache.rollback(transaction)
             transaction.commit()
+            # Clear transaction cache to update default_factory
+            transaction.cache.clear()
 
         if not update:
             pool.setup()
@@ -288,6 +301,11 @@ def load_module_graph(graph, pool, update=None, lang=None):
         while modules_todo:
             (module, to_delete) = modules_todo.pop()
             convert.post_import(pool, module, to_delete)
+
+        if update:
+            # Ensure cache is clear for other instances
+            Cache.clear_all()
+            Cache.refresh_pool(transaction)
     logger.info('all modules loaded')
 
 
@@ -337,6 +355,8 @@ def register_classes():
 
 def load_modules(
         database_name, pool, update=None, lang=None, activatedeps=False):
+    # Do not import backend when importing module
+    from trytond import backend
     res = True
     if update:
         update = update[:]
@@ -345,15 +365,15 @@ def load_modules(
 
     def _load_modules(update):
         global res
-        TableHandler = backend.get('TableHandler')
         transaction = Transaction()
 
-        with transaction.connection.cursor() as cursor:
+        with transaction.set_context(_no_trigger=True), \
+                transaction.connection.cursor() as cursor:
             # Migration from 3.6: remove double module
             old_table = 'ir_module_module'
             new_table = 'ir_module'
-            if TableHandler.table_exist(old_table):
-                TableHandler.table_rename(old_table, new_table)
+            if backend.TableHandler.table_exist(old_table):
+                backend.TableHandler.table_rename(old_table, new_table)
 
             # Migration from 4.0: rename installed to activated
             cursor.execute(*ir_module.select(ir_module.name,
@@ -374,7 +394,7 @@ def load_modules(
                 cursor.execute(*ir_module.select(ir_module.name,
                         where=ir_module.state.in_(('activated', 'to upgrade',
                                 'to remove'))))
-            module_list = [name for (name,) in cursor.fetchall()]
+            module_list = [name for (name,) in cursor]
             graph = None
             while graph is None:
                 module_list += update
@@ -387,26 +407,28 @@ def load_modules(
 
             load_module_graph(graph, pool, update, lang)
 
+            Configuration = pool.get('ir.configuration')
+            Configuration(1).check()
+
             if update:
                 cursor.execute(*ir_module.select(ir_module.name,
                         where=(ir_module.state == 'to remove')))
-                fetchall = cursor.fetchall()
-                if fetchall:
-                    for (mod_name,) in fetchall:
-                        # TODO check if ressource not updated by the user
-                        cursor.execute(*ir_model_data.select(
+                for mod_name, in cursor:
+                    res = False
+                    # TODO check if ressource not updated by the user
+                    with transaction.connection.cursor() as cursor_delete:
+                        cursor_delete.execute(*ir_model_data.select(
                                 ir_model_data.model, ir_model_data.db_id,
                                 where=(ir_model_data.module == mod_name),
                                 order_by=ir_model_data.id.desc))
-                        for rmod, rid in cursor.fetchall():
+                        for rmod, rid in cursor_delete:
                             Model = pool.get(rmod)
                             Model.delete([Model(rid)])
-                        Transaction().connection.commit()
-                    cursor.execute(*ir_module.update([ir_module.state],
-                            ['not activated'],
-                            where=(ir_module.state == 'to remove')))
-                    Transaction().connection.commit()
-                    res = False
+                    transaction.connection.commit()
+                cursor.execute(*ir_module.update([ir_module.state],
+                        ['not activated'],
+                        where=(ir_module.state == 'to remove')))
+                transaction.connection.commit()
 
                 Module = pool.get('ir.module')
                 Module.update_list()
